@@ -1,7 +1,6 @@
 # Pod::Text::Termcap -- Convert POD data to ASCII text with format escapes.
-# $Id: Termcap.pm,v 1.6 2001/11/28 05:44:09 eagle Exp $
 #
-# Copyright 1999, 2001 by Russ Allbery <rra@stanford.edu>
+# Copyright 1999, 2001, 2002, 2004, 2006 by Russ Allbery <rra@stanford.edu>
 #
 # This program is free software; you may redistribute it and/or modify it
 # under the same terms as Perl itself.
@@ -30,8 +29,7 @@ use vars qw(@ISA $VERSION);
 # Don't use the CVS revision as the version, since this module is also in Perl
 # core and too many things could munge CVS magic revision strings.  This
 # number should ideally be the same as the CVS revision in podlators, however.
-$VERSION = 1.06;
-
+$VERSION = 2.03;
 
 ##############################################################################
 # Overrides
@@ -39,22 +37,25 @@ $VERSION = 1.06;
 
 # In the initialization method, grab our terminal characteristics as well as
 # do all the stuff we normally do.
-sub initialize {
-    my $self = shift;
+sub new {
+    my ($self, @args) = @_;
     my ($ospeed, $term, $termios);
+    $self = $self->SUPER::new (@args);
 
-    # The default Term::Cap path won't work on Solaris.
-    $ENV{TERMPATH} = "$ENV{HOME}/.termcap:/etc/termcap"
-        . ":/usr/share/misc/termcap:/usr/share/lib/termcap";
+    # $ENV{HOME} is usually not set on Windows.  The default Term::Cap path
+    # may not work on Solaris.
+    my $home = exists $ENV{HOME} ? "$ENV{HOME}/.termcap:" : '';
+    $ENV{TERMPATH} = $home . '/etc/termcap:/usr/share/misc/termcap'
+                           . ':/usr/share/lib/termcap';
 
     # Fall back on a hard-coded terminal speed if POSIX::Termios isn't
     # available (such as on VMS).
     eval { $termios = POSIX::Termios->new };
     if ($@) {
-        $ospeed = '9600';
+        $ospeed = 9600;
     } else {
         $termios->getattr;
-        $ospeed = $termios->getospeed;
+        $ospeed = $termios->getospeed || 9600;
     }
 
     # Fall back on the ANSI escape sequences if Term::Cap doesn't work.
@@ -64,32 +65,30 @@ sub initialize {
     $$self{NORM} = $$term{_me} || "\e[m";
 
     unless (defined $$self{width}) {
-        $$self{width} = $ENV{COLUMNS} || $$term{_co} || 80;
-        $$self{width} -= 2;
+        $$self{opt_width} = $ENV{COLUMNS} || $$term{_co} || 80;
+        $$self{opt_width} -= 2;
     }
 
-    $self->SUPER::initialize;
+    return $self;
 }
 
 # Make level one headings bold.
 sub cmd_head1 {
-    my $self = shift;
-    local $_ = shift;
-    s/\s+$//;
-    $self->SUPER::cmd_head1 ("$$self{BOLD}$_$$self{NORM}");
+    my ($self, $attrs, $text) = @_;
+    $text =~ s/\s+$//;
+    $self->SUPER::cmd_head1 ($attrs, "$$self{BOLD}$text$$self{NORM}");
 }
 
 # Make level two headings bold.
 sub cmd_head2 {
-    my $self = shift;
-    local $_ = shift;
-    s/\s+$//;
-    $self->SUPER::cmd_head2 ("$$self{BOLD}$_$$self{NORM}");
+    my ($self, $attrs, $text) = @_;
+    $text =~ s/\s+$//;
+    $self->SUPER::cmd_head2 ($attrs, "$$self{BOLD}$text$$self{NORM}");
 }
 
 # Fix up B<> and I<>.  Note that we intentionally don't do F<>.
-sub seq_b { my $self = shift; return "$$self{BOLD}$_[0]$$self{NORM}" }
-sub seq_i { my $self = shift; return "$$self{UNDL}$_[0]$$self{NORM}" }
+sub cmd_b { my $self = shift; return "$$self{BOLD}$_[1]$$self{NORM}" }
+sub cmd_i { my $self = shift; return "$$self{UNDL}$_[1]$$self{NORM}" }
 
 # Output any included code in bold.
 sub output_code {
@@ -103,11 +102,18 @@ sub wrap {
     local $_ = shift;
     my $output = '';
     my $spaces = ' ' x $$self{MARGIN};
-    my $width = $$self{width} - $$self{MARGIN};
-    my $code = "(?:\Q$$self{BOLD}\E|\Q$$self{UNDL}\E|\Q$$self{NORM}\E)";
+    my $width = $$self{opt_width} - $$self{MARGIN};
+
+    # $codes matches a single special sequence.  $char matches any number of
+    # special sequences preceeding a single character other than a newline.
+    # We have to do $shortchar and $longchar in variables because the
+    # construct ${char}{0,$width} didn't do the right thing until Perl 5.8.x.
+    my $codes = "(?:\Q$$self{BOLD}\E|\Q$$self{UNDL}\E|\Q$$self{NORM}\E)";
+    my $char = "(?:$codes*[^\\n])";
+    my $shortchar = $char . "{0,$width}";
+    my $longchar = $char . "{$width}";
     while (length > $width) {
-        if (s/^((?:$code?[^\n]){0,$width})\s+//
-            || s/^((?:$code?[^\n]){$width})//) {
+        if (s/^($shortchar)\s+// || s/^($longchar)//) {
             $output .= $spaces . $1 . "\n";
         } else {
             last;
@@ -115,10 +121,9 @@ sub wrap {
     }
     $output .= $spaces . $_;
     $output =~ s/\s+$/\n\n/;
-    $output;
+    return $output;
 }
 
-
 ##############################################################################
 # Module return value and documentation
 ##############################################################################
@@ -128,7 +133,7 @@ __END__
 
 =head1 NAME
 
-Pod::Text::Color - Convert POD data to ASCII text with format escapes
+Pod::Text::Termcap - Convert POD data to ASCII text with format escapes
 
 =head1 SYNOPSIS
 
@@ -158,7 +163,11 @@ termcap information.
 
 =head1 SEE ALSO
 
-L<Pod::Text>, L<Pod::Parser>, L<Term::Cap>
+L<Pod::Text>, L<Pod::Simple>, L<Term::Cap>
+
+The current version of this module is always available from its web site at
+L<http://www.eyrie.org/~eagle/software/podlators/>.  It is also part of the
+Perl core distribution as of 5.6.0.
 
 =head1 AUTHOR
 
@@ -166,7 +175,7 @@ Russ Allbery <rra@stanford.edu>.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 1999, 2001 by Russ Allbery <rra@stanford.edu>.
+Copyright 1999, 2001, 2002, 2004, 2006 by Russ Allbery <rra@stanford.edu>.
 
 This program is free software; you may redistribute it and/or modify it
 under the same terms as Perl itself.
